package MT::Plugin::MovaMotion;
use strict;
use MT;
use MT::Plugin;
use MT::Author;
use File::Basename;
use Image::Size;

our $VERSION = '0.6';
our $SCHEMA_VERSION = '0.12';

use base qw( MT::Plugin );

@MT::Plugin::MovaMotion::ISA = qw( MT::Plugin );

my $plugin = new MT::Plugin::MovaMotion( {
    id   => 'MovaMotion',
    key  => 'movamotion',
    name => 'MovaMotion',
    description => '<MT_TRANS phrase=\'_PLUGIN_DESCRIPTION\'>',
    version => $VERSION,
    schema_version => $SCHEMA_VERSION,
    author_name => 'Junnama Noda',
    author_link => 'http://junnama.alfasado.net/online/',
    l10n_class => 'MovaMotion::L10N',
    settings => new MT::PluginSettings ( [
        [ 'movamotion_account', { Default => '' } ],
        [ 'movamotion_password', { Default => '' } ],
        [ 'movamotion_server', { Default => '' } ],
        [ 'movamotion_protocol', { Default => 'pop3' } ],
        [ 'movamotion_outdir', { Default => 'files' } ],
        [ 'movamotion_address', { Default => '' } ],
        [ 'movamotion_imagewidth', { Default => '480' } ],
        [ 'movamotion_imagemarkup', { Default => '<p class="image">,</p>' } ],
    ] ),
    blog_config_template => 'blog_config.tmpl',
} );

MT->add_plugin( $plugin );

sub init_registry {
    my $plugin = shift;
    $plugin->registry( {
        callbacks => {
            'MT::App::CMS::template_param.edit_author',
                => \&_edit_author_param,
            'cms_post_save.author',
                => \&_post_save_author,
        },
        tags => {
            block => {
                'MovaMotionBlogCtx' => \&_blog_ctx,
                'MovaMotionEntryCtx' => \&_entry_ctx,
                'IfUserAgentIsDoCoMo?' => \&_hdlr_if_docomo,
                'IfUserAgentIsAU?' => \&_hdlr_if_au,
                'IfUserAgentIsSoftBank?' => \&_hdlr_if_softbank,
                'IfUserAgentIsiPhone?' => \&_hdlr_if_iphone,
                'IfUserAgentIsMobile?' => \&_hdlr_if_mobile,
            },
            function => {
                'AccesskeyChar'    => \&_hdlr_access_key,
                'UserAgent?' => \&_hdlr_useragent,
            },
            modifier => {
                'reply2author' => \&_fltr_reply2author,
                'url2link' => \&_fltr_url2link,
                'remove_anchor' => \&_fltr_remove_anchor,
            },
        },
        object_types => {
            'author' => {
                'mobile_id' => 'string meta',
                'mobile_address' => 'string meta',
                'mail_token' => 'string meta',
            },
        },
        tasks => {
            'MovaMotion' => {
                label     => 'Post entry from e-mail',
                frequency => 299,
                code      => \&_entry_from_email,
            },
        },
   } );
}

sub _blog_ctx {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance();
    my $blog = $app->blog;
    local $ctx->{ __stash }{ blog } = $blog;
    local $ctx->{ __stash }{ blog_id } = $blog->id if defined $blog;
    my $tokens = $ctx->stash( 'tokens' );
    my $builder = $ctx->stash( 'builder' );
    my $out = $builder->build( $ctx, $tokens, $cond );
}

sub _entry_ctx {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance();
    my $blog = $app->blog;
    my $id = $app->param( 'id' );
    my $entry = MT::Entry->load( $id );
    local $ctx->{ __stash }{ blog } = $blog;
    local $ctx->{ __stash }{ blog_id } = $blog->id if defined $blog;
    local $ctx->{ __stash }{ entry } = $entry if defined $entry;
    my $tokens = $ctx->stash( 'tokens' );
    my $builder = $ctx->stash( 'builder' );
    my $out = $builder->build( $ctx, $tokens, $cond );
}

sub _hdlr_useragent {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance();
    return $app->get_header( 'User-Agent' );
}

sub _hdlr_if_docomo {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance();
    my @browswer = split( m!/!, $app->get_header( 'User-Agent' ) );
    my $ua = $browswer[0];
    if ( $ua eq 'DoCoMo' ) {
        return 1;
    } else {
        return 0;
    }
}

sub _hdlr_if_au {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance();
    my @browswer = split( m!/!, $app->get_header( 'User-Agent' ) );
    my $ua = $browswer[0];
    if ( $ua =~ /UP\.Browser/ ) {
        return 1;
    } else {
        return 0;
    }
}

sub _hdlr_if_softbank {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance();
    my @browswer = split( m!/!, $app->get_header( 'User-Agent' ) );
    my $ua = $browswer[0];
    if ( ( $ua eq 'SoftBank' ) || ( $ua eq 'Vodafone' ) ) {
        return 1;
    } else {
        return 0;
    }
}

sub _hdlr_if_iphone {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance();
    my $ua = $app->get_header( 'User-Agent' );
    if ( ( $ua =~ /iPhone/ ) || ( $ua =~ /iPod/ ) ) {
        return 1;
    } else {
        return 0;
    }
}

sub _hdlr_if_mobile {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance();
    return _if_mobile( $app );
}

sub _fltr_remove_anchor {
    my ( $text, $arg, $ctx ) = @_;
    $text =~ s!^<a.*?>(.*?)</a>$!$1!;
    return $text;
}

sub _fltr_reply2author {
    my ( $text, $arg, $ctx ) = @_;
    my $app = MT->instance;
    my $sessid = $app->param( 'sessid' );
    my $blog_id = $app->param( 'blog_id' );
    my $url = $app->base . $app->uri( mode => 'view',
                                      args => { blog_id => $blog_id,
                                                sessid => $sessid } );
    for my $reply ( $text =~ m/(@[a-zA-Z0-9]{1,})/g ) {
        my $username = $reply;
        $username =~ s/^@//;
        my $author = MT::Author->load( { name => $username } );
        if ( defined $author ) {
            my $html = "<a href=\"$url&amp;author=$username\">$reply</a>";
            $text =~ s/$reply/$html/;
        }
    }
    return $text;
}

sub _fltr_url2link {
    my ( $text, $arg, $ctx ) = @_;
    my $app = MT->instance;
    my $is_mobile = _if_mobile( $app );
    for my $http ( $text =~ m/s?https?:\/\/[-_.!~*'()a-zA-Z0-9;\/?:\@&=+\$,%#]+/g ) {
        my $url;
        if ( $is_mobile ) {
            $url = $app->base . $app->uri( mode => 'redirect',
                                           args => { url => $http } );
        } else {
            $url = $http;
        }
        my $link = $http;
        $link = substr( $link, 0, 35 ) . '...' if ( length ( $link ) > 35 );
        my $html = "<a href=\"$url\">$link</a>";
        $text =~ s/$http/$html/;
    }
    return $text;
}

sub _hdlr_access_key {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance;
    my $accesskey = $args->{ 'num' };
    my @browswer = split( m!/!, $app->get_header( 'User-Agent' ) );
    my $ua = $browswer[0];
    if ( $ua eq 'DoCoMo' ) {
        my $code;
        if ( $app->config->PublishCharset eq 'Shift_JIS' ) {
            if ( $accesskey eq '0' ) {
                $code = 63888;
            } else {
                $code = 63878 + $accesskey;
            }
            return "&#$code;";
        } else {
            if ( $accesskey eq '0' ) {
                $code = 'E6EB';
            } elsif ( $accesskey eq '9' ) {
                $code = 'E6EA';
            } else {
                $accesskey ++;
                $code = 'E6E' . $accesskey;
            }
            return "&#x$code;";
        }
    } elsif ( $ua =~ /UP\.Browser/ ) {
        my $code;
        if ( $accesskey eq '0' ) {
            return '<img localsrc="325" alt="0" />';
        } else {
            my $code = 179 + $accesskey;
            return "<img localsrc=\"$code\" alt=\"$accesskey\" />";
        }
    } elsif ( ( $ua eq 'SoftBank' ) || ( $ua eq 'Vodafone' ) ) {
         if ( $accesskey eq '0' ) {
             return '$FE';
         } elsif ( $accesskey eq '1' ) {
             return '$F<';
         } elsif ( $accesskey eq '2' ) {
             return '$F=';
         } elsif ( $accesskey eq '3' ) {
             return '$F>';
         } elsif ( $accesskey eq '4' ) {
             return '$F?';
         } elsif ( $accesskey eq '5' ) {
             return '$F@';
         } elsif ( $accesskey eq '6' ) {
             return '$FA';
         } elsif ( $accesskey eq '7' ) {
             return '$FB';
         } elsif ( $accesskey eq '8' ) {
             return '$FC';
         } elsif ( $accesskey eq '9' ) {
             return '$FD';
         }
    }
    return "[$accesskey]";
}

sub _if_mobile {
    my $app = shift;
    my $ua = $app->get_header( 'User-Agent' );
    if ( ( $ua =~ /iPhone/ ) || ( $ua =~ /iPod/ ) ) {
        return 1;
    }
    my @browswer = split( m!/!, $ua );
    $ua = $browswer[0];
    if ( $ua eq 'DoCoMo' ) {
        return 1;
    }
    if ( ( $ua eq 'SoftBank' ) || ( $ua eq 'Vodafone' ) ) {
        return 1;
    }
    if ( $ua =~ /UP\.Browser/ ) {
        return 1;
    }
    return 0;
}

sub _post_save_author {
    my ( $cb, $app, $obj, $original ) = @_;
    my $mobile_address = $app->param( 'mobile_address' );
    my $mail_token = $app->param( 'mail_token' );
    $obj->mobile_address( $mobile_address );
    $obj->mail_token( $mail_token );
    $obj->save or die $obj->errstr;
}

sub _edit_author_param {
    my ( $cb, $app, $param, $tmpl ) = @_;
    if ( $app->param( 'id' ) ) {
        my $author = MT::Author->load( $app->param( 'id' ) );
        if ( defined $author ) {
            $param->{ mobile_address } = $author->mobile_address;
            $param->{ mail_token } = $author->mail_token;
        }
    }
    my $pointer_field = $tmpl->getElementById( 'email' );
    my $nodeset = $tmpl->createElement( 'app:setting', 
                                        { id    => 'mail_token',
                                          label => $plugin->translate( 'Secret Token' ) ,
                                          required => 0 } );
    my $innerHTML = '<div class="textarea-wrapper" style="width:50% !important"><input type="password" class="full-width" name="mail_token" id="mail_token" class="full-width short" value="<$mt:var name="mail_token" escape="html"$>" /></div>';
    $nodeset->innerHTML ( $innerHTML );
    $tmpl->insertAfter ( $nodeset, $pointer_field );
    $pointer_field = $tmpl->getElementById( 'email' );
    $nodeset = $tmpl->createElement( 'app:setting', 
                                     { id    => 'mobile_address',
                                       label => $plugin->translate( 'Moblie E-mail' ) ,
                                          required => 0 } );
    $innerHTML = '<div class="textarea-wrapper" style="width:50% !important"><input type="text" class="full-width" name="mobile_address" id="mobile_address" class="full-width short" value="<$mt:var name="mobile_address" escape="html"$>" /></div>';
    $nodeset->innerHTML ( $innerHTML );
    $tmpl->insertAfter ( $nodeset, $pointer_field );
}

sub _entry_from_email {
    my $iter = MT::Blog->load_iter( undef );
    my $res;
    while ( my $blog = $iter->() ) {
        $res = _post_from_email( $blog );
    }
    return 1;
}

sub _post_from_email {
    my $blog = shift;
    my $user_id = shift;
    my $app = MT->instance();
    my $do;
    eval { require Net::POP3;
           require MIME::Parser; };
    unless ( $@ ){
        use MT::FileMgr;
        my $fmgr = MT::FileMgr->new( 'Local' ) or die MT::FileMgr->errstr;
        use File::Path;
        use Encode;
        use Encode qw/encode decode/;
        my $charset = $app->config( 'PublishCharset' );
        $charset = 'utf8' if $charset eq 'UTF-8';
        $charset = 'euc-jp' if $charset eq 'EUC-JP';
        $charset = 'shift-jis' if $charset eq 'Shift_JIS';
        my $tempdir = $app->config( 'TempDir' );
        use File::Temp qw/ tempfile tempdir /; 
        my $blog_id = $blog->id;
        my $site_path = _site_path( $blog );
        my $site_url = _site_url( $blog );
        mkdir( $tempdir, 0777 ) unless ( -d $tempdir );
        my $server = $plugin->get_config_value( 'movamotion_server', 'blog:'. $blog_id );
        my $account = $plugin->get_config_value( 'movamotion_account', 'blog:'. $blog_id );
        my $passwd = $plugin->get_config_value( 'movamotion_password', 'blog:'. $blog_id );
        my $protocol = $plugin->get_config_value( 'movamotion_protocol', 'blog:'. $blog_id );
        my $outdir = $plugin->get_config_value( 'movamotion_outdir', 'blog:'. $blog_id );
        my $imagewidth = $plugin->get_config_value( 'movamotion_imagewidth', 'blog:'. $blog_id );
        my $imagemarkup = $plugin->get_config_value( 'movamotion_imagemarkup', 'blog:'. $blog_id );
        my @imagemarkups = split( /,/, $imagemarkup ) if $imagemarkup;
        $outdir = File::Spec->catfile ( $blog->site_path, $outdir );
        return if ( (! $server ) || (! $account ) || (! $passwd ) );
        mkdir( $outdir, 0777 ) unless ( -d $outdir );
        my $pop3 = Net::POP3->new( $server ) or $app->log( "Could not open pop3. (Server:$server)" );
        next unless ( defined $pop3 );
        my $login = ( lc( $protocol ) eq 'apop' ) ? 'apop' : 'login';
        my $count = $pop3->$login( $account, $passwd ) if ( ( $account ) && ( $passwd ) && ( $login ) );
        return unless $count;
        my $messages = $pop3->list() if $count;
        my @entries;
        for my $id ( sort ( keys %$messages ) ) {
            my $message = $pop3->get( $id );
            my $parser = new MIME::Parser;
            my $workdir = tempdir ( DIR => $tempdir );
            $parser->output_dir( $workdir );
            my $entity = $parser->parse_data( $message );
            my $header = $entity->head;
            my $from = $header->get( 'From' );
            my $to = $header->get( 'to' );
            my $token = $1 if ( $to =~ m/\+(.*)\@/ );
            my $subject = $header->get( 'Subject' );
            $subject = encode( $charset, decode( 'MIME-Header', $subject ) );
            $from = encode( $charset , decode( 'MIME-Header', $from ) );
            next unless $from;
            $from =~ s/\n//sg;
            $subject =~ s/\n//sg if $subject;
            $from = $1 if ( $from =~ /<(.{1,})>/ );
            my @authors = MT::Author->search_by_meta( mobile_address => $from );
            unless ( @authors ) {
                $pop3->delete( $id );
                rmtree( [ $workdir ] );
                next;
            }
            my $author = $authors[0];
            if ( (! $author->is_superuser ) && (! $author->permissions( $blog_id )->can_post ) ) {
                $pop3->delete( $id );
                rmtree( [ $workdir ] );
                next;
            }
            if ( $user_id ) {
                next if ( $user_id != $author->id );
                next if ( ( $token ) && ( $token != $author->mail_token ) );
            }
            my $body; my $links; my $photos = 1;
            if ( $entity->is_multipart ) {
                opendir( DIR, $workdir );
                my @files = readdir( DIR );
                closedir( DIR );
                for my $file ( @files ) {
                    unless ( $file =~ /^\./ ) {
                        my $tilda = quotemeta( '%7E' );
                        my $newfile = $file;
                        $newfile =~ s/$tilda//g;
                        my $file_path = File::Spec->catfile ( $workdir, $file );
                        if ( $newfile ne $file ) {
                            my $out_path = File::Spec->catfile ( $workdir, $newfile );
                            $fmgr->rename ( $file_path, $out_path );
                            $file = $newfile;
                            $file_path = $out_path;
                        }
                        if ( $file =~ /^msg\-[0-9]{1,}\-[0-9]{1,}\.txt/ ) {
                            $body = $fmgr->get_data( $file_path );
                        } else {
                            my @suffix = split( /\./, $file );
                            my $file_ext = lc ( pop( @suffix ) );

                            if ( ( $file_ext =~ /^jpe?g$/i ) || ( $file_ext =~ /^gif$/i ) || ( $file_ext =~ /^png$/i ) ) {
                                my $out = File::Spec->catfile ( $outdir, $file );
                                $out = _uniq_fileneme( $out );
                                $fmgr->rename ( $file_path, $out );
                                my %asset_elements = ( file => $out,
                                                       blog_id => $blog_id,
                                                       author_id => $author->id, );
                                my $asset = _save_asset( $app, $blog, %asset_elements );
                                my %param = ( Width => $imagewidth );
                                my ( $thumbnail, $w, $h ) = $asset->thumbnail_file( %param );
                                my $regex_path = quotemeta( $site_path );
                                $thumbnail =~ s/^$regex_path/$site_url/;
                                my $asset_id = $asset->id;
                                my $asset_url = $asset->url;
                                my $alt = $plugin->translate( 'Photo' ) . "($photos)";
                                my $link = "<a href=\"$asset_url\">";
                                $link .= "<img alt=\"\" src=\"$thumbnail\" width=\"$w\" height=\"$h\" /></a>";
                                if ( $imagemarkup ) {
                                    $link = $imagemarkups[0] . $link . $imagemarkups[1];
                                }
                                $link = "<form mt:asset-id=\"$asset_id\">" . $link .'</form>';
                                $links .= $link;
                                $photos++;
                            }
                        }
                    }
                }
            } else {
                $body = $entity->bodyhandle;
                $body = $body->as_string;
            }
            $body = encode( $charset, decode( 'iso-2022-jp', $body ) );
            my $entry = MT::Entry->new;
            $entry->title( $subject );
            $entry->text( $body );
            $entry->text_more( $links );
            $entry->author_id( $author->id );
            $entry->blog_id( $blog_id );
            $entry->status( MT::Entry::RELEASE() );
            $entry->save or die $entry->errstr;
            $do = 1;
            $pop3->delete( $id );
            rmtree( [ $workdir ] );
            push ( @entries, $entry );
        }
        $pop3->quit;
        if ( @entries ) {
            require MT::WeblogPublisher;
            my $pub = MT::WeblogPublisher->new();
            for my $entry ( @entries ) {
                $pub->rebuild_entry( Entry => $entry,
                                     BuildDependencies => 1,
                                     NoIndexes => 1 );
            }
            $pub->rebuild_indexes( Blog => $blog );
        }
    }
    return $do;
}

our %mime_type = (
    'gif'   =>  'image/gif',
    'jpeg'  =>  'image/jpeg',
    'jpg'   =>  'image/jpeg',
    'jpe'   =>  'image/jpeg',
    'png'   =>  'image/png',
);

sub _save_asset {
    my ( $app, $blog, %asset_elements ) = @_;
    my $blog_id = $blog->id;
    my $file_path = $asset_elements{ file };
    my $author_id = $asset_elements{ author_id };
    my $label = $asset_elements{ label };
    my $description = $asset_elements{ description };
    my $parant = $asset_elements{ parant };
    my $basename = basename( $file_path );
    my @suffix = split( /\./, $file_path );
    my $file_ext = lc ( pop( @suffix ) );
    my $mime_type = $mime_type{ $file_ext };
    my $asset_class = 'file'; my $is_image;
    if ( ( $file_ext =~ /^jpe?g$/i ) || ( $file_ext =~ /^gif$/i ) || ( $file_ext =~ /^png$/i ) ) {
        $asset_class = 'image';
        $is_image = 1;
    }
    require MT::Asset;
    my $asset_pkg = MT::Asset->handler_for_file( $basename );
    my $asset;
    if ( $asset_class eq 'image' ) {
        $asset_pkg->isa( 'MT::Asset::Image' );
    }
    my $site_url = _site_url( $blog );
    my $site_path = quotemeta ( _site_path( $blog ) );
    $file_path =~ s/^$site_path/%r/;
    my $url = $file_path;
    $url =~ s!\\!/!g if ( $^O eq 'MSWin32' );
    $asset = $asset_pkg->new();
    $asset->blog_id( $blog_id );
    $asset->url( $url );
    $asset->file_path( $file_path );
    $asset->file_name( $basename );
    $asset->mime_type( $mime_type );
    $asset->file_ext( $file_ext );
    $asset->class( $asset_class );
    $asset->created_by( $author_id );
    $asset->modified_by( $author_id );
    if ( $parant ) {
        $asset->parent( $parant );
    }
    my ( $w,$h );
    if ( $asset_class eq 'image' ) {
        ( $w,$h ) = imgsize( $file_path );
        $asset->image_width( $w );
        $asset->image_height( $h );
    }
    if ( $label ) {
        $asset->label( $label );
    }
    if ( $description ) {
        $asset->description( $description );
    }
    $asset->save or die $asset->errstr;
    my @fdatas = stat( $file_path );
    my $bytes = $fdatas[7];
    my $res = _upload_callback( $app, $asset->file_path, $asset->url, $bytes,
                                $asset, $blog, $w, $h, $asset_class, $is_image );
    return $asset;
}

sub _upload_callback {
    my ( $app, $file, $url, $bytes, $asset, $blog, $h, $w, $type, $is_image ) = @_; 
    if ($is_image) {
        $app->run_callbacks(
            'cms_upload_file.' . $asset->class,
            File  => $file,
            file  => $file,
            Url   => $url,
            url   => $url,
            Size  => $bytes,
            size  => $bytes,
            Asset => $asset,
            asset => $asset,
            Type  => 'image',
            type  => 'image',
            Blog  => $blog,
            blog  => $blog
        );
        $app->run_callbacks(
            'cms_upload_image',
            File       => $file,
            file       => $file,
            Url        => $url,
            url        => $url,
            Size       => $bytes,
            size       => $bytes,
            Asset      => $asset,
            asset      => $asset,
            Height     => $h,
            height     => $h,
            Width      => $w,
            width      => $w,
            Type       => 'image',
            type       => 'image',
            ImageType  => $type,
            image_type => $type,
            Blog       => $blog,
            blog       => $blog
        );
    } else {
        $app->run_callbacks(
            'cms_upload_file.' . $asset->class,
            File  => $file,
            file  => $file,
            Url   => $url,
            url   => $url,
            Size  => $bytes,
            size  => $bytes,
            Asset => $asset,
            asset => $asset,
            Type  => 'file',
            type  => 'file',
            Blog  => $blog,
            blog  => $blog
        );
    }
    return 1;
}

sub _uniq_fileneme {
    my $file = shift;
    return $file unless ( -f $file );
    my @suffix = split( /\./,$file );
    my $file_ext = pop( @suffix );
    my $base = $file;
    $base =~ s/(.{1,})\.$file_ext$/$1/;
    my $i = 0;
    do { $i++;
         $file = $base . '_' . $i . '.' . $file_ext;
       } while ( -e $file );
    return $file;
}

sub _site_path {
    my $blog = shift;
    my $site_path = $blog->archive_path;
    $site_path = $blog->site_path unless $site_path;
    if ( $site_path =~ /(.*)\/$/ ) {
        $site_path = $1;
    }
    if ( $^O eq 'MSWin32' ) {
        $site_path =~ s!/!\\!g;
        if ( $site_path =~ /(.*)\\$/ ) {
            $site_path = $1;
        }
    }
    return $site_path;
}

sub _site_url {
    my $blog = shift;
    my $site_url = $blog->site_url;
    if ( $site_url =~ /(.*)\/$/ ) {
        $site_url = $1;
    }
    return $site_url;
}

1;